<?php

namespace Drupal\content_planner\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Implements DashboardController class.
 */
class DashboardController extends ControllerBase
{

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The dashboard block plugin manager.
   *
   * @var \Drupal\content_planner\DashboardBlockPluginManager
   */
  protected $dashboardBlockPluginManager;

  /**
   * Defines an interface for a service which has the current account stored.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    $instance = parent::create($container);
    $instance->database = $container->get('database');
    $instance->configFactory = $container->get('config.factory');
    $instance->dashboardSettingsService = $container->get('content_planner.dashboard_settings_service');
    $instance->dashboardBlockPluginManager = $container->get('content_planner.dashboard_block_plugin_manager');
    $instance->currentUser = $container->get('current_user');

    return $instance;
  }

  /**
   * Show dashboard.
   *
   * @return array
   *   The content planner dashboard render array.
   */
  public function showDashboard($dashboard_id = 'master')
  {

    $build = [];

    $config_factory = \Drupal::service('config.factory');

    // Ensure the 'master' dashboard exists and is configured.
    $dashboard_config = $config_factory->getEditable('content_planner.dashboards.' . $dashboard_id);
    if ($dashboard_id === 'master' && !$dashboard_config->get('configuration')) {
        $default_master_dashboard_config = [
            'name' => 'Master Dashboard',
            'widgets' => [], // Optionally predefine some widgets.
            'roles' => ['authenticated'], // Specify roles that should have access to the master dashboard.
        ];
        $dashboard_config->set('configuration', $default_master_dashboard_config)->save();
        //$this->messenger()->addMessage($this->t('The master dashboard was not found and has been created with default settings.'));
    }

    // Load the updated or existing dashboard configuration.
    $dashboard_configuration = $dashboard_config->get('configuration');

    // Ensure the "master" dashboard tab is enabled in the tabs configuration.
    $tabs_config = $config_factory->getEditable('content_planner.dashboard_tabs_settings');
    $tabs = $tabs_config->get('dashboards') ?: [];
    if (!array_key_exists('master', array_column($tabs, 'dashboard_id'))) {
        $tabs[] = ['dashboard_id' => 'master', 'enabled' => TRUE, 'weight' => -10];
        $tabs_config->set('dashboards', $tabs)->save();
    }


    // If there are no blocks enabled, display error message.
    if (
      empty($dashboard_configuration['widgets'])
    ) {
      /*
      $this->messenger()->addMessage(
        $this->t(
          'No widgets configured for this dashboard.',
        )
      );*/
    }

    // Get registered Plugins.
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();

    // Check if the current user has permission to configure dashboards
    if ($this->currentUser->hasPermission('administer content planner dashboard settings')) {
      // Create a link to the dashboard settings form
      $config_link = Link::createFromRoute(
        $this->t('Configure Dashboard'),
        'content_planner.dashboard_settings',
        ['dashboard_id' => $dashboard_id],
        ['attributes' => ['class' => ['button', 'button--secondary', 'button--small']]]
      )->toString();

      // Add the link to the build array
      $build['config_link'] = [
        '#type' => 'markup',
        '#markup' => $config_link,
        '#weight' => -10, // Adjust the weight to place the link at the top
      ];
    }

    if (!empty($dashboard_configuration['widgets']) && $dashboard_configuration['widgets']) {
      // Build blocks.
      $block_builds = $this->buildBlocks($dashboard_id, $dashboard_configuration, $plugins);
      $build['widgets'] = [
        '#theme' => 'content_planner_dashboard',
        '#full_width' => isset($dashboard_configuration['width']) ? $dashboard_configuration['width'] : FALSE,
        '#blocks' => $block_builds,
        '#attached' => [
          'library' => ['content_planner/dashboard'],
        ],
      ];
    }

    return $build;
  }

  /**
   * Build blocks.
   *
   * @param array $blocks
   *   The blocks to render.
   * @param array $plugins
   *   The block plugins.
   *
   * @return array
   *   Array of content block render arrays.
   */
  protected function buildBlocks($dashboard_id, array &$dashboard_configuration, array &$plugins)
  {
    $block_builds = [];

    // Loop over every enabled block.
    foreach ($dashboard_configuration['widgets'] as $block) {

      // If a Dashboard Block plugin exists.
      if (array_key_exists($block['plugin_id'], $plugins)) {

        $dashboard_destination = Url::fromRoute(
          'content_planner.dashboard',
          [
            'dashboard_id' => $dashboard_id,
          ]
        );
        /** @var \Drupal\content_planner\DashboardBlockInterface $instance */
        try {
          $instance = $this->dashboardBlockPluginManager->createInstance($block['plugin_id'], $block);
          if ($this->currentUser->hasPermission('administer content planner dashboard settings')) {
            $has_permission = TRUE;
          } else {
            $has_permission = FALSE;
          }

          // Build block render array.
          if ($block_build = $instance->build()) {


            $block_builds[$block['uuid']] = [
              '#theme' => 'content_planner_dashboard_block',
              '#uuid' => $block['uuid'],
              '#dashboard_id' => $dashboard_id,
              '#destination_url' => $dashboard_destination->toString(),
              '#css_id' => str_replace('_', '-', $block['plugin_id']),
              '#block_id' => $block['plugin_id'],
              '#name' => (isset($block['title']) && $block['title']) ? $block['title'] : $instance->getName(),
              '#has_permission' => $has_permission,
              '#block' => $block_build,
              '#weight' => $block['weight'],
              '#colspan_class' => !empty($block['colspan']) ? $block['colspan'] : ''
            ];
          }
        } catch (\Exception $e) {
          $block_builds[$block['uuid']] = [
            '#theme' => 'content_planner_dashboard_block',
            '#uuid' => $block['uuid'],
            '#dashboard_id' => $dashboard_id,
            '#destination_url' => $dashboard_destination->toString(),
            '#css_id' => str_replace('_', '-', $block['plugin_id']),
            '#block_id' => $block['plugin_id'],
            '#name' => (isset($block['title']) && $block['title']) ? $block['title'] : $instance->getName(),
            '#has_permission' => $has_permission,
            '#block' => '(!) Widget cannot be rendered, because it not longer exist or the providing module is missing. Remove it in the dashboard configuration.',
            '#weight' => $block['weight'],
            '#colspan_class' => !empty($block['colspan']) ? $block['colspan'] : ''
          ];
        }



      }
    }

    return $block_builds;
  }

}
