<?php

namespace Drupal\content_planner;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Implements DashboardSettingsService class.
 */
class DashboardSettingsService
{

  /**
   * Config name.
   */
  const CONFIG_NAME = 'content_planner.dashboard_settings';

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new DashboardSettingsService object.
   */
  public function __construct(ConfigFactoryInterface $config_factory)
  {
    $this->configFactory = $config_factory;
  }

  /**
   * Get dashboard configurations.
   *
   * @param string $dashboard_id
   *   The dashboard id to retrieve the config for.
   *
   * @return array
   *   The dashboard configuration.
   */
  public function getDashboardConfigurations($dashboard_id = 'master')
  {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    return $config->get('configuration') ?: [];
  }

  /**
   * Save dashboard configurations.
   *
   * @param string $dashboard_id
   *   The dashboard id to save the config for.
   * @param array $configurations
   *   The dashboard configurations to save.
   */
  public function saveDashboardConfigurations($dashboard_id, array $configurations)
  {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    // Save configurations for the specified dashboard
    $config->set('configuration', $configurations)->save();
  }

  public function deleteDashboardConfigurations($dashboard_id)
  {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $config->delete();
  }

  /**
   * Gets the complete settings config of the module.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   The complete settings config of the module.
   */
  public function getSettings()
  {
    return $this->configFactory->get(self::CONFIG_NAME);
  }

  /**
   * Gets the widget configuration for a dashboard.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   *
   * @return array
   *   The plugin settings.
   */
  public function getWidgetConfigurations($dashboard_id, $uuid)
  {
    $settings = $this->getDashboardConfigurations($dashboard_id);
    if (!isset($settings['widgets'])) {
      return [];
    }

    $widgets = $settings['widgets'];
    if (!isset($widgets[$uuid])) {
      return [];
    }

    return $widgets[$uuid];
  }

  /**
   * Sets a widget configuration.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   * @param array $data
   *   The array of data.
   *
   * @return array
   *   The plugin settings.
   */
  public function setWidgetConfigurations($dashboard_id, $uuid, $data)
  {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $configuration = $config->get('configuration');
    $configuration['widgets'][$uuid] = $data;

    $config->set('configuration', $configuration)->save();

    return $configuration['widgets'][$uuid];
  }

  /**
   * Deletes a widget from a dashboard.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   * @param array $data
   *   The array of data.
   *
   * @return array
   *   The plugin settings.
   */
  public function deleteWidget($dashboard_id, $uuid)
  {
    // Delete the config.
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $configuration = $config->get('configuration');
    unset($configuration['widgets'][$uuid]);

    $config->set('configuration', $configuration)->save();

    return $configuration['widgets'][$uuid];
  }

  /**
   * Get Block configurations.
   *
   * @return array
   *   All block configurations.
   */
  public function getBlockConfigurations()
  {
    if ($settings = $this->getSettings()) {
      if ($block_configurations = $settings->get('blocks')) {
        return $block_configurations;
      }
    }

    return [];
  }

  /**
   * Get the configuration of a specific block.
   *
   * @param string $block_id
   *   The block id to retrieve the config for.
   *
   * @return array|mixed
   *   The block configuration.
   */
  public function getBlockConfiguration($block_id)
  {
    if ($block_configurations = $this->getBlockConfigurations()) {
      if (array_key_exists($block_id, $block_configurations)) {
        return $block_configurations[$block_id];
      }
    }

    return [];
  }

  /**
   * Save configuration of a specific block.
   *
   * @param string $block_id
   *   The block id to save the config for.
   * @param array $configuration
   *   The config to save.
   *
   * @return bool
   *   TRUE if config was found and saved, FALSE otherwise.
   */
  public function saveBlockConfiguration($block_id, array $configuration)
  {
    if ($block_configurations = $this->getBlockConfigurations()) {
      if (array_key_exists($block_id, $block_configurations)) {
        $block_configurations[$block_id] = $configuration;
        $this->saveBlockConfigurations($block_configurations);
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Save all block configurations.
   *
   * @param array $configuration
   *   The config to save.
   */
  public function saveBlockConfigurations(array $configuration)
  {
    $this->configFactory->getEditable(self::CONFIG_NAME)
      ->set('blocks', $configuration)
      ->save();
  }

  /**
   * Gets all available dashboards.
   *
   * @return array
   *   An associative array of all dashboard IDs.
   */
  public function getAllDashboards()
  {
    $allConfigNames = $this->configFactory->listAll('content_planner.dashboards.');
    $dashboards = [];

    foreach ($allConfigNames as $configName) {
      // Extract the dashboard ID from the config name
      $dashboardId = str_replace('content_planner.dashboards.', '', $configName);

      // Optionally, load dashboard name or other properties here
      $dashboardConfig = $this->configFactory->get($configName);

      // Store the dashboard ID and any other details in the result array
      $dashboards[$dashboardId] = $dashboardConfig->get('configuration');
    }

    return $dashboards;
  }


}
