<?php

namespace Drupal\content_planner\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;

class DashboardCreateForm extends FormBase
{

  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'content_planner_dashboard_create_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $form['dashboard_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dashboard Name'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the name of the dashboard.'),
      '#maxlength' => 255,
    ];

    // Machine name
    $form['dashboard_id'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Dashboard machine name'),
      '#maxlength' => 255,
      '#description' => $this->t('A unique machine-readable name for this dashboard. It must only contain lowercase letters, numbers, and underscores.'),
      '#machine_name' => [
        'exists' => [$this, 'dashboardExists'],
        'source' => ['dashboard_name'],
        'replace_pattern' => '[^a-z0-9_]+',
        'replace' => '_',
      ],
      '#required' => TRUE,
    ];

    $role_options = [];
    $default_roles = []; // Array to hold role machine names as default values.
    $roles = Role::loadMultiple();
    foreach ($roles as $role_id => $role) {
      if ($role_id != 'anonymous') {
        $role_options[$role_id] = $role->label();
        $default_roles[] = $role_id; // Add role ID to default selected roles.
      }
    }

    $form['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $role_options,
      '#default_value' => $default_roles, // Set the default value to all role IDs.
      '#description' => $this->t('Select the roles that can view this dashboard.'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create Dashboard'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * Check if the dashboard already exists.
   *
   * @param string $dashboard_id
   *   The dashboard machine name.
   *
   * @return bool
   *   TRUE if the dashboard exists, FALSE otherwise.
   */
  public function dashboardExists($dashboard_id)
  {
    $dashboard_configurations = \Drupal::service('content_planner.dashboard_settings_service')->getDashboardConfigurations($dashboard_id);
    return !empty($dashboard_configurations);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $dashboard_name = $form_state->getValue('dashboard_name');
    $dashboard_id = $form_state->getValue('dashboard_id');
    $selected_roles = array_filter($form_state->getValue('roles'));

    // Initialize default configurations for the new dashboard.
    $dashboard_configurations = [
      'name' => $dashboard_name,
      'roles' => array_keys($selected_roles), // Save only the selected roles' IDs.
    ];

    // Save the dashboard configurations using the DashboardSettingsService.
    \Drupal::service('content_planner.dashboard_settings_service')->saveDashboardConfigurations($dashboard_id, $dashboard_configurations);

    // Load the existing sorted dashboard settings.
    $config_factory = \Drupal::service('config.factory');
    $dashboard_settings_config = $config_factory->getEditable('content_planner.dashboard_tabs_settings');
    $sorted_dashboards = $dashboard_settings_config->get('dashboards') ?: [];

    // Append the new dashboard to the sorted dashboards list.
    $sorted_dashboards[] = [
      'dashboard_id' => $dashboard_id,
      'enabled' => TRUE, // Assuming you want the new dashboard to be enabled by default.
      'weight' => count($sorted_dashboards), // Place the new dashboard at the end.
      'roles' => array_keys($selected_roles), // Assign selected roles to the new tab as well.
    ];

    // Save the updated sorted dashboards configuration.
    $dashboard_settings_config->set('dashboards', $sorted_dashboards)->save();

    // Redirect to the edit page for the new dashboard.
    $edit_url = Url::fromRoute('content_planner.dashboard_settings', ['dashboard_id' => $dashboard_id]);
    $form_state->setRedirectUrl($edit_url);

    // Optionally, add a confirmation message.
    \Drupal::messenger()->addMessage($this->t('Dashboard %name has been created and added to the Dashboard Tabs.', ['%name' => $dashboard_name]));
  }




}
