<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardBlockPluginManager;
use Drupal\content_planner\DashboardSettingsService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\user\Entity\Role;

/**
 * Defines a form that configures dashboard settings.
 */
class DashboardSettingsForm extends ConfigFormBase
{

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The dashboard block plugin manager.
   *
   * @var \Drupal\content_planner\DashboardBlockPluginManager
   */
  protected $dashboardBlockPluginManager;

  /**
   * DashboardSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings_service
   *   Implements DashboardSettingsService class.
   * @param \Drupal\content_planner\DashboardBlockPluginManager $dashboard_block_manager
   *   Dashboard Block plugin manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    DashboardSettingsService $dashboard_settings_service,
    DashboardBlockPluginManager $dashboard_block_manager
  ) {
    parent::__construct($config_factory);
    $this->dashboardSettingsService = $dashboard_settings_service;
    $this->dashboardBlockPluginManager = $dashboard_block_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('config.factory'),
      $container->get('content_planner.dashboard_settings_service'),
      $container->get('content_planner.dashboard_block_plugin_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'content_planner_dashboard_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames()
  {
    return [
      DashboardSettingsService::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $dashboard_id = 'master')
  {
    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => -10,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save dashboard configuration'),
      '#button_type' => 'primary',
    ];

    $dashboard_configurations = $this->dashboardSettingsService
      ->getDashboardConfigurations($dashboard_id);

    $role_options = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role_id => $role) {
      if ($role_id != 'anonymous') {
        $role_options[$role_id] = $role->label();
      }
    }

    // Add a fieldset for the dashboard settings
    $form['advanced_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
      '#tree' => TRUE,
    ];

    $form['advanced_settings']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dashboard Title'),
      '#default_value' => isset($dashboard_configurations['name']) ? $dashboard_configurations['name'] : "",
      '#required' => TRUE
    ];

    $form['advanced_settings']['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $role_options,
      '#default_value' => isset($dashboard_configurations['roles']) ? $dashboard_configurations['roles'] : [],
      '#description' => $this->t('Select the roles that can view this dashboard.'),
    ];

    $form['advanced_settings']['width'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show dashboard in full width.'),
      '#default_value' => isset($dashboard_configurations['width']) ? $dashboard_configurations['width'] : [],
      //'#description' => $this->t('Show dashboard in full width.'),
    ];

    $form['advanced_settings']['dashboard_id'] = [
      '#type' => 'hidden',
      '#title' => $this->t('Dashboard ID'),
      '#default_value' => $dashboard_id,
      '#required' => TRUE
    ];



    if (!empty($dashboard_configurations['widgets']) && $dashboard_configurations['widgets'] != null) {
      $active_widgets = json_encode(
        $dashboard_configurations['widgets'],
        JSON_FORCE_OBJECT,
      );
    } else {
      $active_widgets = [];
    }

    $form['widgets'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Widgets configuration'),
      '#tree' => TRUE,
    ];

    $form['active_widgets'] = [
      '#type' => 'textarea',
      '#id' => 'active-widgets-field',
      '#default_value' => $active_widgets,
    ];

    $categorized_plugins = $this->getCategorizedPlugins();

    $widgets_data = [];
    foreach ($categorized_plugins as $category => $plugins) {
      foreach ($plugins as $plugin_id => $plugin_name) {
        $widgets_data[$category][$plugin_id] = [
          'plugin_id' => $plugin_id,
          'title' => $plugin_name,
        ];
      }
    }

    // Prepare the data for JS
    $settings = [
      'dashboard' => [
        'widgets' => $widgets_data,
      ],
    ];
    $form['#attached']['drupalSettings'] = $settings;

    $form['#attached']['library'][] = 'content_planner/content_planner_react_app';
    $form['widgets']['react_app'] = [
      '#type' => 'markup',
      '#markup' => '<div id="root"></div>',
    ];

    $form['#attached']['library'][] = 'content_planner/dashboard';

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    // Retrieve the whole submitted values.
    $values = $form_state->getValues();

    // Access advanced settings directly from the nested structure.
    $dashboard_id = $values['advanced_settings']['dashboard_id'];
    $dashboard_name = $values['advanced_settings']['name'];
    $dashboard_width = $values['advanced_settings']['width'];

    // Fetch the configurations for the selected dashboard.
    $dashboard_configurations['name'] = $dashboard_name;
    $dashboard_configurations['width'] = $dashboard_width;

    // Process roles.
    $selected_roles = array_filter($values['advanced_settings']['roles']);
    // Since roles are checkboxes, use array_keys to get selected roles.
    $dashboard_configurations['roles'] = array_keys($selected_roles);

    // Save configurations for the selected dashboard.
    $this->dashboardSettingsService->saveDashboardConfigurations(
      $dashboard_id,
      $dashboard_configurations
    );

    // Widgets configurations are stored in a different part of the form.
    // Assuming 'active_widgets' is directly under $form and not nested within 'widgets'.
    $activeWidgetsJson = $values['active_widgets'];

    // Decode the JSON string to an associative array.
    $activeWidgets = json_decode($activeWidgetsJson, TRUE);
    if ($activeWidgets) {
      foreach ($activeWidgets as $widget_config) {
        $widget_config['dashboard_id'] = $dashboard_id;
        $this->dashboardSettingsService->setWidgetConfigurations(
          $dashboard_id,
          $widget_config['uuid'],
          $widget_config
        );
      }
    }

    // Display a message and redirect.
    $this->messenger()->addMessage(
      $this->t('Dashboard %dashboard has been updated.', ['%dashboard' => $dashboard_name])
    );

    $form_state->setRedirect('content_planner.dashboard', ['dashboard_id' => $dashboard_id]);
  }


  /**
   * Get active widgets for a dashboard.
   *
   * @param string $dashboard_id
   *   Id of the dashboard.
   *
   * @return array
   */
  protected function getActiveWidgets($dashboard_id)
  {
    $blocks = $this->dashboardSettingsService->getDashboardConfigurations($dashboard_id);
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();

    $active_widgets = [];

    foreach ($blocks as $block_id => $block) {
      if (array_key_exists($block_id, $plugins)) {
        $active_widgets[] = [
          'id' => $block_id,
          'title' => (isset($block['title']) && $block['title']) ? $block['title'] : $plugins[$block_id]['name'],
          'plugin_name' => $plugins[$block_id]['name'],
          'weight' => $block['weight'],
          'colspan' => !empty($block['colspan']) ? $block['colspan'] : '',
        ];
      }
    }

    usort($active_widgets, function ($a, $b) {
      return $a['weight'] - $b['weight'];
    });

    return $active_widgets;
  }

  /**
   * Get categorized plugins.
   *
   * @return array
   */
  protected function getCategorizedPlugins()
  {
    // Get registered Plugins.
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();
    uksort($plugins, function ($a, $b) {
      return strnatcmp($a, $b);
    });

    // Initialize categories array
    $categorized_plugins = [];

    foreach ($plugins as $plugin_id => $plugin) {
      // Get the category from the plugin definition
      if (isset($plugin['category'])) {
        $id_category = $plugin['category'];
      } else {
        $id_category = "other";
      }

      if (!isset($categorized_plugins[$id_category])) {
        $categorized_plugins[$id_category] = [];
      }

      $categorized_plugins[$id_category][$plugin_id] = $plugin['name'];
    }

    return $categorized_plugins;
  }

}
