<?php

namespace Drupal\content_planner\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\content_planner\DashboardSettingsService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\user\Entity\Role;
use Drupal\Core\Link;
use Drupal\Core\Url;

class DashboardTabsSettingsForm extends ConfigFormBase
{

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  public static function create(ContainerInterface $container)
  {
    $instance = parent::create($container);
    $instance->dashboardSettingsService = $container->get('content_planner.dashboard_settings_service'); // Replace 'your_service_name' with the actual service name defined in yourmodule.services.yml
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames()
  {
    return [
      'content_planner.dashboard_tabs_settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'dashboard_tabs_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state)
  {
    // Loadd all available dashboards
    $allDashboards = $this->dashboardSettingsService->getAllDashboards();

    // Load tabs from config
    $config = $this->config('content_planner.dashboard_tabs_settings');
    $tabs = $config->get('dashboards') ?: [];


    // Initialize dashboardSettings with all dashboards and default settings
    $dashboardTabSettings = [];
    foreach ($allDashboards as $dashboard_id => $dashboardInfo) {
      $dashboardTabSettings[$dashboard_id] = [
        'dashboard_id' => $dashboard_id,
        'enabled' => false, // Default to disabled
        'weight' => PHP_INT_MAX, // High weight to appear at the end if not configured
        'name' => $dashboardInfo['name'], // Use name from allDashboards data
        'roles' => $dashboardInfo['roles'], // Use roles from allDashboards data
      ];
    }

    // Override defaults with any configured dashboard settings
    foreach ($tabs as $dashboard) {
      // Ensure 'name' from $allDashboards is used if not set in $dashboard
      $dashboard['name'] = $allDashboards[$dashboard['dashboard_id']]['name'];
      $dashboard['roles'] = $allDashboards[$dashboard['dashboard_id']]['roles'];
      $dashboardTabSettings[$dashboard['dashboard_id']] = $dashboard;
    }

    // Sort dashboards by weight
    uasort($dashboardTabSettings, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    $form['dashboards'] = [
      '#type' => 'table',
      '#header' => [
        'Enabled',
        'Dashboard Title',
        'Dashboard ID',
        'Roles',
        'Weight',
        'Actions'
      ],
      '#empty' => $this->t('No dashboards available.'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'dashboards-order-weight',
        ],
      ],
    ];

    $role_labels = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role_id => $role) {
      if ($role_id != 'anonymous') {
        $role_labels[$role_id] = $role->label();
      }
    }

    // Build the form rows based on the sorted dashboard settings
    foreach ($dashboardTabSettings as $dashboard_id => $dashboard) {

      // Get the current request path to set as the destination.
      $current_path = \Drupal::service('path.current')->getPath();
      $destination = ['destination' => $current_path];

      $dashboardUrl = Url::fromRoute('content_planner.dashboard', ['dashboard_id' => $dashboard_id]);
      $dashboardLink = Link::fromTextAndUrl($dashboard['name'], $dashboardUrl)->toString();


      $form['dashboards'][$dashboard_id]['#attributes']['class'][] = 'draggable';
      $form['dashboards'][$dashboard_id]['enabled'] = [
        '#type' => 'checkbox',
        '#default_value' => $dashboard['enabled'],
      ];
      // Add dashboard title
      $form['dashboards'][$dashboard_id]['title'] = [
        '#markup' => $dashboardLink,
      ];

      $form['dashboards'][$dashboard_id]['dashboard_id'] = [
        '#markup' => $dashboard_id,
      ];
      $form['dashboards'][$dashboard_id]['roles'] = [
        '#markup' => implode(', ', array_intersect_key($role_labels, array_flip($dashboardTabSettings[$dashboard_id]['roles'] ?? []))),
      ];
      $form['dashboards'][$dashboard_id]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', ['@title' => $dashboard_id]),
        '#title_display' => 'invisible',
        '#default_value' => $dashboard['weight'] == PHP_INT_MAX ? '' : $dashboard['weight'], // Leave weight blank if it's the placeholder value
        '#attributes' => ['class' => ['dashboards-order-weight']],
      ];
      // Add actions (edit and delete buttons)
      $form['dashboards'][$dashboard_id]['actions'] = [
        '#type' => 'dropbutton',
        '#links' => [
          'edit' => [
            'title' => $this->t('Edit'),
            'url' => Url::fromRoute('content_planner.dashboard_settings', ['dashboard_id' => $dashboard_id], ['query' => $destination]),
            'attributes' => ['class' => ['button--small']], // Attempt to apply a small button class
          ],
          'delete' => [
            'title' => $this->t('Delete'),
            'url' => Url::fromRoute('content_planner.dashboard_delete', ['dashboard_id' => $dashboard_id], ['query' => $destination]),
            'attributes' => ['class' => ['button--small']], // Attempt to apply a small button class
          ],
        ],
      ];
    }

    $form['actions']['add_dashboard'] = [
      '#type' => 'link',
      '#title' => $this->t('Add Dashboard'),
      '#url' => \Drupal\Core\Url::fromRoute('content_planner.dashboard_create_form'),
      '#attributes' => ['class' => ['button']],
      '#weight' => 100,
    ];

    return parent::buildForm($form, $form_state);
  }



  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $values = $form_state->getValue('dashboards');
    $sorted_dashboards = [];

    foreach ($values as $dashboard_id => $details) {
      if ($details['enabled']) {
        $sorted_dashboards[] = [
          'dashboard_id' => $dashboard_id,
          'weight' => $details['weight'],
          'enabled' => (bool) $details['enabled'],
        ];
      }
    }

    usort($sorted_dashboards, function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    $this->config('content_planner.dashboard_tabs_settings')
      ->set('dashboards', $sorted_dashboards)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
